package linprog;

/**
 * Title:        KeplerConjecture 2001
 * Description:  Licensed under GNU GPL http://www.gnu.org/copyleft/gpl.html
 * Copyright:    Copyright (c) 2001, Thomas C. Hales
 * Company:
 * @author Thomas C. Hales
 * @version 2001.0
 */

import java.util.*;


/**
 * A coefficient list is a linear combination of variables.
 *
 */
public class CoefficientList {

  private final HashMap coeff = new HashMap();

  public CoefficientList() {
  }

  /**
   * combines a coefficient with whatever coefficient is already there.
   * precondition V!=null.
   */
  public void addCoeff(Variable V, Interval coefficient) {
    util.Eiffel.assert(V!=null);
    Interval current = (Interval)coeff.get(V);
    if (current==null)
      coeff.put(V,coefficient);
    else
      coeff.put(V,current.add(coefficient));
  }

  /**
   * From the enumeration, all the coefficients can be obtained from getCoefficient().
   * Elements of the enumeration can be safely cast to Variable.
   */
  public Iterator getVariableIterator() {
    return coeff.keySet().iterator();
  }

  /**
   * @param V the variable whose coefficient is requested.
   */
  public Interval getCoefficient(Variable V) {
    return (Interval) coeff.get(V);
  }

  public String toString() {
    StringBuffer B = new StringBuffer();
    for (Iterator I = this.getVariableIterator();I.hasNext();/*--*/) {
      Variable V = (Variable)I.next();
      Interval val = getCoefficient(V);
      B.append(' ');
      if (val.getCenter().signum()>=0)
        B.append('+');
      B.append(val.toString());
      B.append(' ');
      B.append(V.toString());
    }
    return B.toString();
  }

  public static class Test extends util.UnitTest {
    public void test() {
      java.math.BigDecimal u = new java.math.BigDecimal(11.3);
      Variable X = new Variable(u,u);
      Variable Y = new Variable(u,u);
      CoefficientList C = new CoefficientList();
      C.addCoeff(X,new Interval("1.0"));
      C.addCoeff(Y,new Interval("2.0"));
      C.addCoeff(X,new Interval("3.0"));
      Interval near0 = C.getCoefficient(X).subtract(new Interval("4.0"));
      assert(near0.getCenter().signum()==0);
      assert(near0.getRadius().signum()==0);
      near0 = C.getCoefficient(Y).subtract(new Interval("2.0"));
      assert(near0.getCenter().signum()==0);
      assert(near0.getRadius().signum()==0);
    }
  }
}