package linprog;
import graph.*;
import graph.archive.*;
import java.math.BigDecimal;
import java.util.*;

/**
 * Title:        KeplerConjecture 2001
 * Description:  Licensed under GNU GPL http://www.gnu.org/copyleft/gpl.html
 * Copyright:    Copyright (c) 2001, Thomas C. Hales
 *
 * @author
 * @version 2001.0
 */

 /**
  * What I use to control the LPs.
  */
public class Controller {

  private static final Parameters P = ParameterFactory.get();
  private static final Interval target = P.scoreTarget();

  private Controller() {
  }

  private static BigDecimal objective(MetricGraph MG) {
    GraphLP gLP = new GraphLP(MG);
    BigDecimal BB=null;
    for (int counter=0;counter<5;counter++) {
      try {
          BB = gLP.runObjectiveSigma();
          System.out.println("objective = "+BB);
        return BB;
      }
      catch (Exception e) {
        //e.printStackTrace();
        System.out.println("retry....");
      }
    }
  return null;
  }

  private static boolean splitQuad(MetricGraph MG,Stack stackMG) {
    graph.Face F = MG.getOriginalQuad();
    if (F==null) {
      stackMG.push(MG);
      return false;
    }
    System.out.println(".............quad branch.........");
    graph.Vertex V = F.getAny();
    MetricGraph M;
    M = MG.divideQuad(F,V,P.eM(),Parameters.I_sqrt8approx);
    stackMG.push(M);
    //new CoordinatesDemo(M,"fist divide");
    M = MG.divideQuad(F,F.next(V,1),P.eM(),Parameters.I_sqrt8approx);
    stackMG.push(M);
    //new CoordinatesDemo(M,"second divide");
    M=MG.fourQuarters(F,P.eM().min(),P.I_sqrt8approx.max());
    stackMG.push(M);
    //new CoordinatesDemo(M,"quarters");
    MG.setFATQUAD(F);
    stackMG.push(MG);
    //new CoordinatesDemo(MG,"fat");
    return true;
  }

   private static boolean updateHeight(MetricGraph MG) {
    GraphLP gLP = new GraphLP(MG);
    boolean success = false;
    BigDecimal A = new BigDecimal("2.005");
    for (int counter=0;counter<5;counter++) {
      try {
          for (Enumeration E = MG.vertexEnumeration();E.hasMoreElements();/*--*/) {
            graph.Vertex V = (graph.Vertex)E.nextElement();
            if (Miscellaneous.lessThan(MG.getMaxHeight(V),new BigDecimal("2.13")))
              continue;
            BigDecimal BB = gLP.runObjectiveV(V,ilog.cplex.example.CpxProblem.Maximize,A);
            if (Miscellaneous.lessThan(BB,MG.getMaxHeight(V))) {
              MG.setMaxHeight(V,BB);
              success = true;
            }
          }
          break;
      }
      catch (Exception e) {
      }
    }
    return success;
  }

  private static BigDecimal calculateMaxHeight(MetricGraph G) {
    GraphLP gLP = new GraphLP(G);
    BigDecimal A = new BigDecimal("2.0");
    for (int counter=0;counter<5;counter++) {
      try {
          for (Enumeration E = G.vertexEnumeration();E.hasMoreElements();/*--*/) {
            graph.Vertex V = (graph.Vertex)E.nextElement();
            BigDecimal BB = gLP.runObjectiveV(V,ilog.cplex.example.CpxProblem.Maximize,new BigDecimal("2.0005"));
            if (Miscellaneous.lessThan(A,BB) && Miscellaneous.lessEqual(BB,new BigDecimal("2.51005")))
              A = BB;
          }
          return A;
      }
      catch (Exception e) {}
    }
    return P.eM().max();
  }

  private static BigDecimal maxSlopeOnUpright(MetricGraph G) {
    GraphLP gLP = new GraphLP(G);
    BigDecimal A = new BigDecimal("2.0");
    Object[] VF = G.getUpright();
    if (VF==null)
      return null;
    graph.Vertex V = (graph.Vertex)VF[0];
    graph.Face F = (graph.Face)VF[1];
    for (int counter=0;counter<5;counter++) {
      try {
          for (int i=0;i<V.size();i++) {
            graph.Vertex W = V.next(F,i).next(V,1);
            BigDecimal BB = gLP.runObjectiveE(V,W,ilog.cplex.example.CpxProblem.Maximize);
            if (Miscellaneous.lessThan(A,BB))
              A = BB;
          }
          return A;
      }
      catch (Exception e) {}
    }
    return P.eM().max();
  }

  /**
   * Break into two cases top>=6.25 or top<=6.25.
   */
  private static boolean branchQRTET(MetricGraph MG, Stack stackMG) {
    //1. get tetrahedron to split.
    Face F=null;
    for (Enumeration E = MG.faceEnumeration();E.hasMoreElements();/*--*/) {
      Face Ft = (Face)E.nextElement();
      if (MG.isQRTET(Ft) && !MG.isBIGQRTET(Ft) && !MG.isSmallQRTET(Ft)) {
        F = Ft;
        break;
      }
    }
    if (F==null) return false;
    System.out.println("....................branching now...");
    MetricGraph MG1 = (MetricGraph)MG.clone();
    MG1.setBIGQRTET(F);
    stackMG.push(MG1);
    MG.setSmallQRTET(F);
    stackMG.push(MG);
    return true;
  }


  public static void run(int index) {
    graph.Graph G = getQuad(index);
    Stack stackMG = new Stack();
    util.Eiffel.assert(G!=null);
    int counter =0;
    MetricGraph MG = new MetricGraph(G);
    stackMG.push(MG);
    while (stackMG.size()>0) {
      System.out.println("stacksize = "+stackMG.size()+" on "+index);
      MG = (MetricGraph)stackMG.pop();
      BigDecimal value = objective(MG);
      util.Eiffel.assert(value!=null);
      if (Miscellaneous.lessThan(value,target.min()))
        continue;
      if (0== (counter++ % 3)) {
        updateHeight(MG);
      }
      if (MG.mightHaveOriginalQuad() && splitQuad(MG,stackMG))
        continue;
      if (branchQRTET(MG,stackMG))
        continue;
      // report on unfinished case.
      System.out.println("tough quad case found  "+index+" score "+value);
      new CoordinatesDemo(MG,"tough quad case "+index);
      System.out.println("Formatted example:");
      System.out.println(graph.Formatter.toArchiveString(MG));
      try {
        IO.toFile(Parameters.ineqPath+"completed.txt","toughcase2["+index+"]="+value+";  //"+
          LPdata.getGlobalCounter()+"   "+calculateMaxHeight(MG)+" "+maxSlopeOnUpright(MG)+"\n",true);
      }
      catch (Exception e) {
        e.printStackTrace();
      }
      //break; // Put in to exit early on first unsolvable case.
    }
    System.out.println("Case "+index+ " completed.");
  }

  private static graph.Graph getQuad(int i) {
    String archive = graph.archive.graphDispatch.getArchiveString(graphDispatch.QUAD,i);
    return graph.Graph.getInstance(new Formatter(archive));
  }

  private static void showQuad(int i) {
    new CoordinatesDemo(getQuad(i),"quad "+i);
  }

  private static void findSpecial() {
    int sz = graphDispatch.size(graphDispatch.QUAD);
    System.out.println("QUAD cases ="+sz);
    for (int i=0;i<sz;i++) {
      graph.Graph G = getQuad(i);
      if (G.vertexSize()==12 && G.faceSize() <= 15) {
        boolean good = true;
        for (Enumeration E = G.vertexEnumeration();E.hasMoreElements();/*--*/) {
          graph.Vertex V = (graph.Vertex) E.nextElement();
          if (V.size()<4 || V.size()>5)
            good = false;
          if (V.size()==5 && V.faceCount(3,3)!=5)
            good = false;
          if (V.size()==4 && V.faceCount(3,3)!=2)
            good = false;
        }
        if (good)
          showQuad(i);
      }
    }
  }

  /**
    * test for Metric Graph
    */
  /**
  public void testCode() {
    showQuad(0);
    graph.Face F =null;
    MetricGraph MG = new MetricGraph(getQuad(0));
    for (Enumeration E = MG.faceEnumeration();E.hasMoreElements();) {
      F = (graph.Face) E.nextElement();
      if (F.size()==4)
        break;
    }
    util.Eiffel.assert(F!=null);
    graph.Vertex V = F.getAny();
    MetricGraph MGw = MG.addUpright(F,V,P.eM().min(),P.I_sqrt8approx.max());
    new CoordinatesDemo(MGw," modified 0 ");
    MetricGraph MGww = MGw.moreUpright();
    util.Eiffel.assert(MGww!=null);
    new CoordinatesDemo(MGww," mm 0 ");
    MetricGraph MGwww = MGww.moreUpright();
    util.Eiffel.assert(MGwww!=null);
    new CoordinatesDemo(MGwww," mmm 0");
  }
  **/

  public static void main(String[] args) {
    //fileIO(graphDispatch.DODEC,61);//67
    int sz = graphDispatch.size(graphDispatch.QUAD);
    //findSpecial();
    //System.out.println("eM = "+P.eM());
    //Miscellaneous.fileIO(graphDispatch.QUAD,27);
    int[] toughcase = new int[] //{15, 40, 41, 57, 60, 68, 69, 71, 113, 118, 120, 123, 125, 126,138, 142, 146, 156, 178, 237, 350};
            //{71,126,142,178,350};
            {178};
    for (int i=0;i<sz;i++) {
      if (i!=1679 && i!= 1672 && i!= 1640) // fcc, hcp pent-prism
        run(i);
    }
    System.out.println("done... "+LPdata.getGlobalCounter());


    //MetricGraph MG = new MetricGraph(getQuad(1672));
    //GraphLP gLP = new GraphLP(MG);
    //System.out.println(gLP.toString());
    //Miscellaneous.printSolution(MG,gLP);
    //graph.Formatter F = new graph.Formatter("0 22 3 0 2 1 3 0 3 2 3 0 4 3 3 0 5 4 3 1 2 7 3 1 7 6 3 2 3 8 3 2 8 7 3 3 4 9 3 3 9 8 3 4 5 10 3 4 10 9 3 5 6 10 3 6 7 11 3 6 11 10 3 7 8 11 3 8 9 11 3 9 10 11 3 5 0 12 3 12 0 1 3 12 1 6 3 12 6 5 ");
    //MetricGraph MG = new MetricGraph(graph.Graph.getInstance(F));
    //new CoordinatesDemo(MG,"178 with markup");
    //showQuad(178);


  /*
    MetricGraph MG = new MetricGraph(getQuad(178));
    graph.Face F = MG.getOriginalQuad();
    MG=MG.fourQuarters(F,P.eM().min(),P.I_sqrt8approx.max());
    //new CoordinatesDemo(MG,"howdy");
    updateHeight(MG);
    GraphLP gLP = new GraphLP(MG);
    //util.Eiffel.assert(gLP.quarteredQuads());
    System.out.println(gLP.toString());
    Miscellaneous.printSolution(MG,gLP);
    */



  }



}