package linprog;

/**
 * Title:        KeplerConjecture 2001
 * Description:  Licensed under GNU GPL http://www.gnu.org/copyleft/gpl.html
 * Copyright:    Copyright (c) 2001, Thomas C. Hales
 * Company:
 * @author Thomas C. Hales
 * @version 2001.0
 */
import java.math.BigDecimal;
/**
 * This represents a top-edge in a quasi-regular triangle in a Voronoi cell.  All the
 * data is held in a workspace.  The variable aspect represents the edge
 * along the top of the packing
 * <p>
 * @precondition the top edge has length at most 8.
 * In a saturated packing, each point of a face of a Voronoi cell has distance
 * at most 2 from the center, so the defining vertex of that face has distance
 * at most 4, and any two such vertices have distance at most 8 from each other.
 */
public class Edge extends Variable {

  private static final BigDecimal two = new BigDecimal(2.0);
  private static final BigDecimal eight = new BigDecimal(8.0);
  /**
   *
   */
  public Edge() {
    super(two, eight,"yedge");
  }

  public Edge(String id) {
    super(two,eight,"yedge"+id);
  }
}