package linprog;

/**
 * Title:        KeplerConjecture 2001
 * Description:  Licensed under GNU GPL http://www.gnu.org/copyleft/gpl.html
 * Copyright:    Copyright (c) 2001, Thomas C. Hales
 * Company:
 * @author Thomas C. Hales
 * @version 2001.0
 */
import java.util.*;
import util.UnitTest;

/**
 * This is a generic collections class that could be moved to a generic util package.
 * Implements a sort of associative array with multidimensional keys.
 * A key matches an entry iff the arrays are the same length, and corresponding
 * entries are equal (in the sense of the method "equal").
 */
public class Function implements Cloneable {

  private HashMap map  = new HashMap();

  public Function() {

  }

  public void add(Object[] keylist,Object value) {
    map.put(new Key(keylist),value);
  }

  public Object get(Object[] keylist) {
    return map.get(new Key(keylist));
  }

  /**
   * values of iterator are castable to (Object[])
   */
  public Iterator iterator() {
    Iterator I = new Iterator() {
      Iterator it = map.keySet().iterator();
      public boolean hasNext() {
        return it.hasNext();
      }
      public Object next() {
        return ((Key)it.next()).toArray();
      }
      public void remove() {
        throw new java.lang.UnsupportedOperationException();
      }
    };
    return I;
  }

  public Object clone() {
    Function F = new Function();
    F.map = (HashMap)this.map.clone();
    return F;
  }

  private class Key {
    private Object[] keylist;
    public Key(Object[] keylist) {
      this.keylist = keylist;
    }

    public Object[] toArray() {
      return keylist;
    }

    public boolean equals(Object obj) {
      try {
        Key castObj = (Key) obj;
        if (this.keylist.length != castObj.keylist.length)
          return false;
        for (int i=0;i<this.keylist.length;i++) {
          if (!this.keylist[i].equals(castObj.keylist[i]))
            return false;
        }
        return true;
      }
      catch (Exception e) {
      }
      return false;
    }
    /**
     * overrides Object's hashCode(). Whenever the "equals" method is changed,
     * so must hashCode, if HashMap is to work correctly.
     * The contract is explained in section 20.1.4 of the
     * Java Language Specification, Gosling et al.
     * Look at the source code for HashMap.containsKey to see the use of hashCode.
     */
    public int hashCode() {
      StringBuffer B = new StringBuffer();
      for (int i=0;i<keylist.length;i++) {
        B.append(Integer.toBinaryString(keylist[i].hashCode()));
      }
      return B.toString().hashCode();
    }
  }

  //test code.
  public static class Test extends util.UnitTest {
    public void test1() {
      Function F = new Function();
      F.add(new Object[] {"A","B","C"},"ABC out");
      assert(F.map.size()==1);
      F.add(new Object[] {"A","E","F"},"AEF old");
      F.add(new Object[] {"A","E","F"},"AEF new");
      Object g = F.get(new Object[] {"A","B","C"});
      assert("ABC out".equals(F.get(new Object[] {"A","B","C"})));
      assert("AEF new".equals(F.get(new Object[] {"A","E","F"})));
      assert(null == F.get(new Object[] {"A","B","D"}));
      Function C = (Function)F.clone();
      assert("ABC out".equals(C.get(new Object[] {"A","B","C"})));
      assert("AEF new".equals(C.get(new Object[] {"A","E","F"})));
      assert(null == C.get(new Object[] {"A","B","D"}));
      C.add(new Object[] {"R","S","T"},"RST");
      assert(C.map.size()== F.map.size()+1);
      assert(F.get(new Object[]{"R","S","T"})==null);
      assert(C.get(new Object[]{"R","S","T"}).equals("RST"));
    }
  }

  public Collection values() {
    return map.values();
  }

}