import ilog.cplex.example.CpxSparseVector;
import ilog.cplex.example.CpxException;
import ilog.cplex.example.CpxProblem;

public class Lpex1 {
   static void populatebyrow (CpxProblem cplex) throws CpxException
   {
      cplex.setObjSense(CpxProblem.Maximize);

      CpxSparseVector[] cols = new CpxSparseVector[3];
      double[]       obj  = {1.0, 2.0, 3.0};
      double[]       lb   = {0.0, 0.0, 0.0};
      double[]       ub   = {40.0, CpxProblem.Infinity, CpxProblem.Infinity};
      char[]         type = {CpxProblem.Continuous,
                             CpxProblem.Continuous, CpxProblem.Continuous};
      cplex.addCols(obj, cols, lb, ub, type);

      CpxSparseVector[] rows  = new CpxSparseVector[2];
      char[]         sense = {CpxProblem.LessEqual, CpxProblem.LessEqual};
      double[]       rhs   = {20.0, 30.0};
      rows[0] = new CpxSparseVector();
      rows[1] = new CpxSparseVector();
      rows[0].add(0, -1.0);
      rows[0].add(1,  1.0);
      rows[0].add(2,  1.0);
      rows[1].add(0,  1.0);
      rows[1].add(1, -3.0);
      rows[1].add(2,  1.0);
      cplex.addRows(rows, sense, rhs);
   }

   static void populatebycolumn (CpxProblem cplex) throws CpxException
   {
      cplex.setObjSense(CpxProblem.Maximize);

      CpxSparseVector[] rows  = new CpxSparseVector[2];
      char[]         sense = {CpxProblem.LessEqual, CpxProblem.LessEqual};
      double[]       rhs   = {20.0, 30.0};
      cplex.addRows(rows, sense, rhs);

      CpxSparseVector[] cols = new CpxSparseVector[3];
      double[]       obj  = {1.0, 2.0, 3.0};
      double[]       lb   = {0.0, 0.0, 0.0};
      double[]       ub   = {40.0, CpxProblem.Infinity, CpxProblem.Infinity};
      char[]         type = {CpxProblem.Continuous,
                             CpxProblem.Continuous, CpxProblem.Continuous};
      cols[0] = new CpxSparseVector();
      cols[1] = new CpxSparseVector();
      cols[2] = new CpxSparseVector();
      cols[0].add(0, -1.0);
      cols[0].add(1,  1.0);
      cols[1].add(0,  1.0);
      cols[1].add(1, -3.0);
      cols[2].add(0,  1.0);
      cols[2].add(1,  1.0);
      cplex.addCols(obj, cols, lb, ub, type);
   }

   static void populatebynonzero (CpxProblem cplex) throws CpxException
   {
      cplex.setObjSense(CpxProblem.Maximize);

      CpxSparseVector[] rows  = new CpxSparseVector[2];
      char[]         sense = {CpxProblem.LessEqual, CpxProblem.LessEqual};
      double[]       rhs   = {20.0, 30.0};
      cplex.addRows(rows, sense, rhs);

      CpxSparseVector[] cols = new CpxSparseVector[3];
      double[]       obj  = {1.0, 2.0, 3.0};
      double[]       lb   = {0.0, 0.0, 0.0};
      double[]       ub   = {40.0, CpxProblem.Infinity, CpxProblem.Infinity};
      char[]         type = {CpxProblem.Continuous,
                             CpxProblem.Continuous, CpxProblem.Continuous};
      cplex.addCols(obj, cols, lb, ub, type);

      int[]    rind = {0, 0, 0, 1, 1, 1};
      int[]    cind = {0, 1, 2, 0, 1, 2};
      double[] val  = {-1.0, 1.0, 1.0, 1.0, -3.0, 1.0};
      cplex.setCoefs(rind, cind, val);
   }


   static void usage() {
      System.out.println("usage: lpex1 -X");
      System.out.println("   where X is one of the following");
      System.out.println("      r    generate problem by row");
      System.out.println("      c    generate problem by column");
      System.out.println("      n    generate problem by nonzero");
   }

   public static void main(String[] args) {
      if ( args.length != 1         ||
           args[0].length() != 2    ||
           args[0].charAt(0) != '-'   ) {
         usage();
         return;
      }

      try {
         CpxProblem cplex = new CpxProblem();
         cplex.setIntParam(1035, 1);

         switch ( args[0].charAt(1) ) {
            case 'r': populatebyrow(cplex);
                      break;
            case 'c': populatebycolumn(cplex);
                      break;
            case 'n': populatebynonzero(cplex);
                      break;
            default:  usage();
                      cplex.dispose();
                      return;
         }

         cplex.primopt();

         int ncols = cplex.getNCols();
         int nrows = cplex.getNRows();

         double[] x     = new double[ncols];
         double[] dj    = new double[ncols];
         double[] pi    = new double[nrows];
         double[] slack = new double[nrows];

         cplex.getX(x);
         cplex.getPi(pi);
         cplex.getDj(dj);
         cplex.getSlack(slack);

         System.out.println("Solution status = " + cplex.getStatus());
         System.out.println("Solution value  = " + cplex.getObj());

         for (int j = 0; j < ncols; ++j) {
            System.out.println("Column " + j +
                               " Value = " + x[j] +
                               " Reduced cost = " + dj[j]);
         }

         for (int i = 0; i < nrows; ++i) {
            System.out.println("Row " + i +
                               " Pi Slack = " + slack[i] +
                               " Pi = " + pi[i]);
         }

         cplex.write("lpex1.lp");
         cplex.dispose();
      }
      catch (CpxException e) {
         System.out.println(e.getMessage());
      }
      catch (Throwable t) {
         System.out.println("caught general exception");
      }
   }
}
