import ilog.cplex.example.CpxException;
import ilog.cplex.example.CpxProblem;

public class Lpex2 {
   static void usage() {
      System.out.println("usage: lpex2 <filename> <algorithm>");
      System.out.println("   where <filename> is a file with extension");
      System.out.println("      MPS, SAV, or LP (lower case is allowed)");
      System.out.println("   and <algorithm> is on of the letters");
      System.out.println("      p    primal simplex         (CPXprimopt)");
      System.out.println("      d    dual simplex           (CPXdualopt)");
      System.out.println("      b    barrier                (CPXbaropt)");
      System.out.println("      h    barrier with crossover (CPXhybbaropt)");
      System.out.println("      n    network simples        (CPXhybnetopt)");
      System.out.println("      m    MIP optimizer          (CPXmipopt)");
   }

   public static void main(String[] args) {
      if ( args.length != 2 ) {
         usage();
         return;
      }

      try {
         CpxProblem cplex = new CpxProblem();
         cplex.setIntParam(1035, 1);

         cplex.read(args[0]);

         switch ( args[1].charAt(0) ) {
            case 'p':   cplex.primopt();
                        break;
            case 'd':   cplex.dualopt();
                        break;
            case 'b':   cplex.baropt();
                        break;
            case 'h':   cplex.hybbaropt();
                        break;
            case 'n':   cplex.hybnetopt();
                        break;
            case 'm':   cplex.mipopt();
                        break;
            default:    usage();
                        cplex.dispose();
                        return;
         }

         System.out.println("solution status " + cplex.getStatus() +
                            ".  Objective value " + cplex.getObj());

         int ncols = cplex.getNCols();
         int nrows = cplex.getNRows();

         boolean  havebasis = true;
         int[]    rstat = new int[nrows];
         int[]    cstat = new int[ncols];
         double[] x     = new double[ncols];

         cplex.getX(x);

         cplex.setIntParam(1035, 0);
         try {
            cplex.getBase(cstat, rstat);
         } catch (CpxException e) {
            if ( e.getNumber() == 1262 ) havebasis = false;
            else throw (e);
         }
         cplex.setIntParam(1035, 1);
         System.out.println("Number of columns: "+ncols);
         for (int j = 0; j < ncols; ++j) {
            if ( havebasis ) {
               System.out.print("Column: " + j + ":  Value = " + x[j] + "  ");
               switch (cstat[j]) {
               case 0:  System.out.println("Nonbasic at lower bound");
                        break;
               case 1:  System.out.println("Basic");
                        break;
               case 2:  System.out.println("Nonbasic at upper bound");
                        break;
               case 3:  System.out.println("Superbasic, or free variable at zero");
                        break;
               default: System.out.println("Bad basis status");
                        break;
               }
            }
            else {
               System.out.println("Column: " + j + ":  Value = " + x[j]);
            }
         }

         //System.out.println("Maximum bound violation = "
                           // + cplex.getDblQuality(1));
         cplex.dispose();
      }
      catch (CpxException e) {
         System.out.println(e.getMessage());
      }
      catch (Throwable t) {
         System.out.println("general exception caught");
      }
   }
}
