package linprog;
import java.util.*;

/**
 * Title:        KeplerConjecture 2001
 * Description:  Licensed under GNU GPL http://www.gnu.org/copyleft/gpl.html
 * Copyright:    Copyright (c) 2001, Thomas C. Hales
 *
 * @author
 * @version 2001.0
 */

 /**
  * ParseIneq builds an Abstract Syntax Tree (AST).  This is a tree of Node objects.
  */

class Node {
  public final static int INEQUALITYLIST =0;
  public final static int DOMAIN=1;
  public final static int INEQUALITY=2;
  public final static int NUMBER=3;
  public final static int SYMBOL=4;
  public final static int SENSE=5;
  public final static int TERM=6;
  public final static int LINCOMBO=7;
  public final static int DOMAIN3=8;
  public final static int DOMAINVAR=9;

  public int type() {
    return type;
  }

  public Node getChild(int index) {
    if (index<0 || index > children.size()) return null;
    return (Node) children.get(index);
  }

  // sometimes the child is a String or Interval.  In such cases there
  // is only one child.
  public Object getObject() {
    return children.get(0);
  }

  public Iterator childIterator() {
    return children.iterator();
  }

  private String tag() {
    switch (type) {
      case INEQUALITYLIST : return "InequalityList";
      case DOMAIN: return "Domain";
      case INEQUALITY: return "Inequality";
      case NUMBER: return "Number";
      case SYMBOL: return "Symbol";
      case SENSE: return "Sense";
      case TERM: return "Term";
      case LINCOMBO: return "LinCombo";
      case DOMAIN3: return "Domain3";
      case DOMAINVAR: return "DomainVar";
      default: return "Unknown";
    }
  }

  private int type;
  private ArrayList children = new ArrayList();

  public Node(int type) {
    this.type = type;
    //System.out.println("node + "+this.toString());
  }
  public void addChild(Object X) {
    children.add(X);
  }

  public String toString() {
    return toString("");
  }

  private String toString(String indent) {
    StringBuffer B = new StringBuffer(indent+"<"+tag()+" size='"+children.size()+"'>\n");
    for (int i=0;i<children.size();i++) {
      try {
        B.append(((Node)children.get(i)).toString(indent+"    "));
      }
      catch (Exception e) {
        B.append(indent+"    "+children.get(i).toString());
      }
    }
    B.append(indent+"</"+tag()+">\n");
    return B.toString();
  }
}