package linprog;
import ilog.cplex.example.CpxProblem;
import java.math.BigDecimal;
import java.util.*;

/**
 * Title:        KeplerConjecture 2001
 * Description:  Licensed under GNU GPL http://www.gnu.org/copyleft/gpl.html
 * Copyright:    Copyright (c) 2001, Thomas C. Hales
 *
 * @author
 * @version 2001.0
 */

/**
 * This is a module style class (no public constructors).
 * It is used to generate standard objective functions.
 * It also generates the inequality for acorvol > target.
 */
public class ObjectiveFactory {

  private ObjectiveFactory() {
  }

  /**
    * Maximize sum sigma
    * varI is a list of sigma for all of the faces.
    */
  public static Objective sigma(Iterator varI) {
    return varLimit(varI,CpxProblem.Maximize,new BigDecimal("0"));
  }


  /**
   * Optimizes any single variable.
   * optimize  V + ( )*threshold, or
   * @precondition sense is CpxProblem.Maximize or CpxProblem.Minimize;
   */
  public static Objective varLimit(Variable V,int sense,BigDecimal thresholdDefault) {
    util.Eiffel.precondition(sense== CpxProblem.Maximize || sense== CpxProblem.Minimize);
    Objective obj = new Objective();
    obj.setSense(sense);
    BigDecimal A = thresholdDefault;
    if (A==null)
      A = (sense==CpxProblem.Maximize ? V.getLowerBound() : V.getUpperBound() );
    obj.addCoeff(V,Parameters.I_one);
    obj.addCoeff(Parameters.threshold,new Interval(A,new BigDecimal(0)));
    return obj;
  }
  /**
   * Optimizes any sum of variables contained in an iterator.
   * optimize   sum V + ( )*threshold, or
   * @precondition sense is CpxProblem.Maximize or CpxProblem.Minimize;
   * @precondition varI.next() castable to Variable.
   */
  public static Objective varLimit(Iterator varI,int sense,BigDecimal thresholdDefault) {
    util.Eiffel.precondition(sense== CpxProblem.Maximize || sense== CpxProblem.Minimize);
    Objective obj = new Objective();
    obj.setSense(sense);
    BigDecimal zero = new BigDecimal(0);
    while (varI.hasNext()) {
      Variable V = (Variable)varI.next();
      if (thresholdDefault==null) {
        BigDecimal A = (sense==CpxProblem.Maximize ? V.getLowerBound() : V.getUpperBound() );
        obj.addCoeff(Parameters.threshold,new Interval(A,zero));
      }
      obj.addCoeff(V,Parameters.I_one);
    }
    if (thresholdDefault!=null) {
      obj.addCoeff(Parameters.threshold,new Interval(thresholdDefault,zero));
    }
    return obj;
  }
}