package linprog;

/**
 * Title:        KeplerConjecture 2001
 * Description:  Licensed under GNU GPL http://www.gnu.org/copyleft/gpl.html
 * Copyright:    Copyright (c) 2001, Thomas C. Hales
 * Company:
 * @author Thomas C. Hales
 * @version 2001.0
 */

import java.math.BigDecimal;

/**
 * Represents a quoin in a truncated Voronoi cell.
 */
public class Quoin extends Variable {

  // zero is a lower bound, because a Quoin is a volume.
  private static final BigDecimal zero = new BigDecimal("0.0");

  /* a rough upper bound comes from, say A[1] Pi since a quoin lies in a
     slice represented by A[1] 2Pi, and in a half-plane, so a factor of 2.
     With edgeMax = 2.51..., A[1] Pi = 0.1229...
  */
  private static final BigDecimal quoinBound = new BigDecimal("0.2");

  public Quoin() {
    super(zero,quoinBound,"quoin");
  }

  public Quoin(String id) {
    super(zero,quoinBound,"quoin"+id);
  }
}