package linprog;



import ilog.cplex.example.CpxSparseVector;
import ilog.cplex.example.CpxException;
import ilog.cplex.example.CpxProblem;

public class Lpex0 {

   static void populatebyrow (CpxProblem cplex) throws CpxException
   {
      cplex.setObjSense(CpxProblem.Maximize);

      CpxSparseVector[] cols = new CpxSparseVector[3];
      double[]       obj  = {1.0, 2.0, 3.0};
      double[]       lb   = {0.0, 0.0, 0.0};
      double[]       ub   = {CpxProblem.Infinity , CpxProblem.Infinity, CpxProblem.Infinity};
      char[]         type = {CpxProblem.Continuous,
                             CpxProblem.Continuous, CpxProblem.Continuous};
      cplex.addCols(obj, cols, lb, ub, type);

      CpxSparseVector[] rows  = new CpxSparseVector[3];
      char[]         sense = {CpxProblem.LessEqual, CpxProblem.LessEqual, CpxProblem.LessEqual};
      double[]       rhs   = {20.0, 30.0, 40.0};
      rows[0] = new CpxSparseVector();
      rows[1] = new CpxSparseVector();
      rows[2] = new CpxSparseVector();
      rows[0].add(0, -1.0);
      rows[0].add(1,  1.0);
      rows[0].add(2,  1.0);
      rows[1].add(0,  1.0);
      rows[1].add(1, -3.0);
      rows[1].add(2,  1.0);
      rows[2].add(0, 1.0);
      rows[2].add(1, 0.0);
      rows[2].add(2, 0.0);
      cplex.addRows(rows, sense, rhs);
   }






   static void usage() {
      System.out.println("usage: lpex0");
   }

   public static void main(String[] args) {

      try {
         CpxProblem cplex = new CpxProblem();
         cplex.setIntParam(1035, 1);
         populatebyrow(cplex);

         cplex.primopt();

         int ncols = cplex.getNCols();
         int nrows = cplex.getNRows();

         double[] x     = new double[ncols];
         double[] dj    = new double[ncols];
         double[] pi    = new double[nrows];
         double[] slack = new double[nrows];

         cplex.getX(x);
         cplex.getPi(pi);
         cplex.getDj(dj);
         cplex.getSlack(slack);

         System.out.println("Solution status = " + cplex.getStatus());
         System.out.println("Solution value  = " + cplex.getObj());

         for (int j = 0; j < ncols; ++j) {
            System.out.println("Column " + j +
                               " Value = " + x[j] +
                               " Reduced cost = " + dj[j]);
         }

         for (int i = 0; i < nrows; ++i) {
            System.out.println("Row " + i +
                               " Pi Slack = " + slack[i] +
                               " Pi = " + pi[i]);
         }

         cplex.write("lpex1.lp");
         cplex.dispose();
      }
      catch (CpxException e) {
         System.out.println(e.getMessage());
      }
      catch (Throwable t) {
         System.out.println("caught general exception");
      }
   }
}