package linprog;

import java.math.BigDecimal;
import java.util.*;
import graph.CoordinatesDemo;
import graph.Formatter;
import util.UnitTest;

/**
 * Title:        KeplerConjecture 2001
 * Description:  Licensed under GNU GPL http://www.gnu.org/copyleft/gpl.html
 * Copyright:    Copyright (c) 2001, Thomas C. Hales
 *
 * @author
 * @version 2001.0
 */

 /**
  * This method contains static methods that were originally part of Controller,
  * but that got separated out, when Controller became too unwieldy.
  */

public class Miscellaneous {

  private Miscellaneous() {
  }

  static void PANIC(MetricGraph MG, GraphLP gLP) {
    printSolution(MG,gLP);
    fileIO(MG);
    new CoordinatesDemo(MG," I'm in a panic!");
    try {
      Thread.sleep(4000000);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    System.exit(0);
  }

  static void fileIO(MetricGraph MG) {
    GraphLP gLP = new GraphLP(MG);
    try {
    IO.toFile(Parameters.ineqPath+"test.txt",gLP.toString(),false/*noappend*/);
    } catch( Exception e) { e.printStackTrace(); }
  }

  static void fileIO(int series,int index) {
    String archive = graph.archive.graphDispatch.getArchiveString(series,index);
    graph.Graph G = graph.Graph.getInstance(new Formatter(archive));
    //new CoordinatesDemo(G,"series "+series+" index "+index);
    fileIO(new MetricGraph(G));

  }

  private static String match(Enumeration E,String start,HashMap sol,boolean exact) {
    StringBuffer b = new StringBuffer();
    b.append("");
    for (/*--*/;E.hasMoreElements();/*--*/) {
      Variable V = (Variable) E.nextElement();
      if ((exact && V.toString().equals(start))||
          (!exact && V.toString().startsWith(start))) {
        Object value = sol.get(V);
        b.append("("+V.toString()+"="+value+")");
      }
    }
    return b.toString();
  }

  private static String matchEnds(Enumeration E,String start,String end,HashMap sol) {
    StringBuffer b = new StringBuffer();
    b.append("");
    while (E.hasMoreElements()) {
      Variable V = (Variable) E.nextElement();
      String S = V.toString();
      if (
          S.startsWith(start) && S.endsWith(end)
          ) {
        Object value = sol.get(V);
        b.append("("+S+"="+value+")");
      }
    }
    return b.toString();
  }



  static void printAngles(MetricGraph MG, GraphLP gLP) {
    for (int i=0;i<MG.faceSize();i++) {
      String start = "alpha_F"+i+"_";
      for (Enumeration E = gLP.variableEnumeration();E.hasMoreElements();/*--*/) {
        Variable V = (Variable) E.nextElement();
        if (V.toString().startsWith(start))
          System.out.print(" "+V.toString());
      }
      System.out.println(" ");
    }
  }

  static void printSolution(MetricGraph MG, GraphLP gLP) {
    HashMap sol = null;
    try {
      gLP.runObjectiveSigma();
      sol = gLP.getSolutionVariable();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    String start="";
    for (int i=0;i<MG.vertexSize();i++) {
      System.out.print(match(gLP.variableEnumeration(),"yv_"+i,sol,true));
      System.out.println(match(gLP.variableEnumeration(),"L("+i+"-",sol,false));
      System.out.print(match(gLP.variableEnumeration(),"yedge("+i+",",sol,false));
      System.out.println(matchEnds(gLP.variableEnumeration(),"yedge(",","+i+")",sol));
      System.out.println(match(gLP.variableEnumeration(),"quoin("+i+"-",sol,false));
      System.out.println(matchEnds(gLP.variableEnumeration(),"alpha","v"+i,sol));
      System.out.println("");
    }
    for (int i=0;i<MG.faceSize();i++) {
      System.out.print(match(gLP.variableEnumeration(),"sigma"+i,sol,false));
      System.out.print(match(gLP.variableEnumeration(),"sol"+i,sol,false));
    }
    System.out.println(" ");
    printAngles(MG,gLP);
  }


  static boolean lessThan(BigDecimal A, BigDecimal B) {
    return A.subtract(B).signum()<0;
  }

  static boolean lessEqual(BigDecimal A, BigDecimal B) {
    return A.subtract(B).signum()<=0;
  }

  static boolean equals(BigDecimal A, BigDecimal B) {
    return A.subtract(B).signum()==0;
  }

  /**
   * used to sort an array largest to smallest, by second element .
   * o1 = {xx,Double}
   * @returns 1 if o1<o2, 0 if equal, -1 if o1>o2.
   */

  final public static Comparator thisComp = new Comparator() {
    public int compare(Object o1, Object o2) {
      double d1 = ((Double)((Object[])o1)[1]).doubleValue();
      double d2 = ((Double)((Object[])o2)[1]).doubleValue();
      return (d1==d2 ? 0 : (d1<d2 ? 1 : -1));
    }
    public boolean equals(Object obj) {
      return super.equals(obj);
    }
  };


  // returns true 1 time in denominator times.
  static boolean coinFlip(int denominator) {
    util.Eiffel.assert(denominator>0);
    double r = Math.random();
    util.Eiffel.assert(r>=0 && r<=1.0);
    return (r*((double) denominator) <= 1.0);
  }

  public static class Test extends util.UnitTest {

        public void testCoinFlip() {
          int number = 0;
          for (int i=0;i<500;i++) {
            if (coinFlip(3))
              number ++;
          }
          assert(number>20);
        }


        public void testSort() {
          Object[] list = new Object[] {new Object[] {"C",new Double(1.0)},
              new Object[]{"B",new Double(2.0)},
                  new Object[]{"A",new Double(3.0)}};
          Arrays.sort((Object[])list,Miscellaneous.thisComp);
          Object[] A = (Object[])list[0];
          assert(A[0].equals("A"),A.toString());
          Object[] B = (Object[])list[1];
          assert(B[0].equals("B"),B.toString());
          Object[] C = (Object[])list[2];
          assert(C[0].equals("C"),C.toString());

        }
  }


}