package linprog;

/**
 * Title:        KeplerConjecture 2001
 * Description:  Licensed under GNU GPL http://www.gnu.org/copyleft/gpl.html
 * Copyright:    Copyright (c) 2001, Thomas C. Hales
 * Company:
 * @author Thomas C. Hales
 * @version 2001.0
 */


/**
 * represents various constant values.
 * The original intention was to bring into this class all of the truncation-dependent
 * constants, so that a change in truncation could easily be made.
 * <p>
 * This is not entirely the case, because truncation-dependent constants appear
 * in the constructors of various Variables: Quoin, and to a lesser
 * extent Vertex and Edge.  If the value of edgeMax is changed, the bounds on these variables should
 * be moved to this class, and then a 1-parameter constructor should be written that
 * gets its parameter from this class.
 * <p>
 * Various inequalities in InequalityFactor are also truncation-dependent
 * (for example, the quoin inequality).
 */
public interface Parameters {


  /**
   * This parameter may need to be changed according to where the inequality files are located.
   */
  static final String ineqPath = "C:\\Documents and Settings\\Thomas Hales\\jbproject\\linprogK98\\src\\linprog\\";


  /**
   * List of external files containing inequalities.
   */
  String[] ineqFiles(); // = new String[] {"ineqDodec.txt"};
  /**
   * The maximum edge length of an edge in a quasi-regular tetrahedron.
   */
  //Interval edgeMax();
  Interval eM();

  /**
   * edgeMax/2
   */
  Interval trunc();

  /**
   * Lower bound on the number of vertices.
   */
  int vertexMinCount();

  /**
   * score target for linear programming minimization
   */
  Interval scoreTarget();

   static final Interval I_mOne = new Interval("-1");
   static final Interval I_zero = new Interval("0");
   static final Interval I_one = new Interval("1");
   static final Interval I_two = new Interval("2");
   static final Interval I_six = new Interval("6");
   final static Interval I_pi = new Interval("3.1415926535897932","1.0e-15");
   // Mathematica: Abs[N[628318530717958647/10^17-2Pi,30]] < 10^-16
   static final Interval I_twopi = new Interval("6.28318530717958647","1.0e-16");
   // Rogers's Voronoi cell bound = 4Pi/(3 dtet);
   final static Interval fourPi_3dtet = new Interval("5.3727540991345877","1.0e-15");

   // doct
   final static Interval I_doct = new Interval("0.7209029495174650","1.0e-14");

   // should be slightly larger than sqrt8.
   final static Interval I_sqrt8approx = new Interval("2.8284271247462","0");
  /**
   * The constant sqrt8;
   * To check the given value in Mathematica, check that this straddles zero:
      (282842712474619/10^14 + {-10^-15,10^-15})^2 - 8;
   * (check done Feb 23, 2001).
   */
   static final Interval I_sqrt8 = new Interval("2.82842712474619","1.0e-15");
   static final java.math.BigDecimal BD_one = new java.math.BigDecimal("1.0");

  // Rogers's packing density.
  static final Interval I_dtet = new Interval("0.7796355700442529","1.0e-15");

  static final Interval I_pt = new Interval("0.055373645668463869730","1.0e-15");

    // force a bounded feasible LP solution.
    static final Variable threshold =
      new Variable(new java.math.BigDecimal("0"),BD_one,"threshold");

    // represents a constant term in an objective function or inequality.
    static final Variable unit = new Variable(BD_one,BD_one,"unit");

    // represents the truncated volume.
    // upper bound is 4 Pi phi0 < 9.0
    static final Variable cvolume = new Variable(new java.math.BigDecimal("0"),
        new java.math.BigDecimal("9.0"),"cvolume");





}