package linprog;

/**
 * Title:        KeplerConjecture 2001
 * Description:  Licensed under GNU GPL http://www.gnu.org/copyleft/gpl.html
 * Copyright:    Copyright (c) 2001, Thomas C. Hales
 *
 * @author
 * @version 2001.0
 */

import ilog.cplex.example.CpxProblem;
import ilog.cplex.example.CpxException;
import java.math.BigDecimal;
import java.math.BigInteger;

public class SmallLP {

  private SmallLP() {
  }

  /**
   * These first inequalities are generic.
   */

  /**
   * Use coefficients and variables sense rhs to create a new inquality.
   */
  static Inequality make(String[] coeff, Variable[] vars, char sense, String rhs) {
    Inequality ineq = new Inequality("make");
    ineq.setRHS(new Interval(rhs));
    ineq.setSense(sense);
    util.Eiffel.assert(coeff.length==vars.length);
    for (int i=0;i<coeff.length;i++) {
      ineq.addCoeff(vars[i],new Interval(coeff[i]));
    }
    ineq.addCoeff(Parameters.threshold,ineq.getRHS());
    return ineq;
  }

  /**
   * creates the inequality sum mult[i] var[i] <= alpha.
   */
  static Inequality makeLT(int[] mult, Variable[] vars,Interval alpha) {
    Interval[] coeff = new Interval[mult.length];
    Inequality ineq = new Inequality("makeLT");
    for (int i=0;i<mult.length;i++) {
      coeff[i]= I(mult[i]);
      ineq.addCoeff(vars[i],coeff[i]);
    }
    ineq.setSense(CpxProblem.LessEqual);
    ineq.setRHS(alpha);
    ineq.addCoeff(Parameters.threshold,ineq.getRHS());
    return ineq;
  }

  /**
   * Make a new varaible with given lower and upper bounds.
   */
  static Variable makeV(String lb,String ub,String name) {
    return new Variable(new BigDecimal(lb),new BigDecimal(ub),name);
  }

  /**
   * int to Interval conversion.
   */
  static Interval I(int k) {
    return new Interval(new Integer(k).toString());
  }

  /**
   * make dihedral
   *  -tauVar < a + b dihVar
   */
  static Inequality makeDih(String a, String b, Variable tauVar, Variable dihVar) {
    return make(new String[]{a,b,"1"},new Variable[]{Parameters.unit,dihVar,tauVar},CpxProblem.GreaterEqual,"0");
  }

  /**
   * *********************************************************
   * This starts the section for upright quarters and other anchored simplices
   * **********************************************************
   */

    // dih variables on uprights
    static final Variable dihQuarter = makeV("0.956","6.2832","dihQuarter");
    static final Variable dihCetc = makeV("1.23","6.2832","dihCetc");
    static final Variable dihFatAnchor = makeV("1.4157","6.2832","dihFatAnchor");
    static final Variable dihLargeGap = makeV("1.65","6.2832","dihLargeGap");
    // tau variables on uprights
    static final Variable tauQuarter = makeV("0","6.0","tauQuarter");
    static final Variable tauCetc = makeV("0","6.0","tauCetc");
    static final Variable tauFatAnchor = makeV("0","6.0","tauFatAnchor");
    static final Variable tauLargeGap = makeV("0","6.0","tauLargeGap");

  static void addStandardAnchored(LPdata data) {
    // add variables.
    data.addVariable(Parameters.threshold,null);
    data.addVariable(Parameters.unit,Parameters.threshold);
    data.addVariable(dihQuarter,Parameters.threshold);
    data.addVariable(dihCetc,Parameters.threshold);
    data.addVariable(dihFatAnchor,Parameters.threshold);
    data.addVariable(dihLargeGap,Parameters.threshold);
    data.addVariable(tauQuarter,Parameters.threshold);
    data.addVariable(tauCetc,Parameters.threshold);
    data.addVariable(tauFatAnchor,Parameters.threshold);
    data.addVariable(tauLargeGap,Parameters.threshold);

    // inequalitiesdihQuarter
    data.addInequality(makeDih("-4.42873","4.16523",tauQuarter,dihQuarter));
    data.addInequality(makeDih("-1.01104","0.78701",tauQuarter,dihQuarter));
    data.addInequality(makeDih("-0.99937","0.77627",tauQuarter,dihQuarter));
    data.addInequality(makeDih("-0.34877","0.21916",tauQuarter,dihQuarter));
    data.addInequality(makeDih("-0.11434","0.05107",tauQuarter,dihQuarter));
    data.addInequality(makeDih("0.07749","-0.07106",tauQuarter,dihQuarter));
    //  inequalitiesdi dihCetc, assuming dih < 2.46
    data.addInequality(makeDih("-3.3407","2.1747",tauCetc,dihCetc));
    data.addInequality(makeDih("-2.945","1.87427",tauCetc,dihCetc));
    data.addInequality(makeDih("-1.5035","0.83046",tauCetc,dihCetc));
    data.addInequality(makeDih("-1.0009","0.48263",tauCetc,dihCetc));
    data.addInequality(makeDih("-0.7787","0.34833",tauCetc,dihCetc));
    data.addInequality(makeDih("-0.4475","0.1694",tauCetc,dihCetc));
    data.addInequality(makeDih("-0.2568","0.0822",tauCetc,dihCetc));
    // inequalities dihFatAnchor, assuming dih < 2.46
    data.addInequality(makeDih("-3.48","2.1747",tauFatAnchor,dihFatAnchor));
    data.addInequality(makeDih("-3.06","1.87427",tauFatAnchor,dihFatAnchor));
    data.addInequality(makeDih("-1.58","0.83046",tauFatAnchor,dihFatAnchor));
    data.addInequality(makeDih("-1.06","0.48263",tauFatAnchor,dihFatAnchor));
    data.addInequality(makeDih("-0.83","0.34833",tauFatAnchor,dihFatAnchor));
    data.addInequality(makeDih("-0.50","0.1694",tauFatAnchor,dihFatAnchor));
    data.addInequality(makeDih("-0.29","0.0822",tauFatAnchor,dihFatAnchor));
  }

  public static BigDecimal uprightTauLP(int quarter,int cetc,int fatanchor,int largegap,String thresh) {
    LPdata data = new LPdata();
    addStandardAnchored(data);
    data.addInequality(makeLT(new int[] {quarter,cetc,fatanchor,largegap},
      new Variable[] {dihQuarter,dihCetc,dihFatAnchor,dihLargeGap},twopimax));
    Objective obj = new Objective();
    obj.setSense(CpxProblem.Minimize);
    obj.addCoeff(tauQuarter,I(quarter));
    obj.addCoeff(tauCetc,I(cetc));
    obj.addCoeff(tauFatAnchor,I(fatanchor));
    obj.addCoeff(Parameters.threshold,new Interval(thresh));
    data.setObjective(obj);
    BigDecimal B = null;
    //System.out.println(data.toString());
    try {
      B = data.optimum();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return B;
  }

  public static void uprightTauLPexample() {
    BigDecimal B = uprightTauLP(3,0,1,1,"1.0");
    //BigDecimal C = tauLP(4,0,new Interval(new BigDecimal(2*Math.PI-0*(0.8638)-1*1.32-0*1.153)),"1.0");
    System.out.println(B);//.add(t8).subtract(target));
    //System.out.println(new Interval(t8.subtract(t6).add(B)).divide(Parameters.I_pt));
  }



  /**
   * *********************************************************
   * This starts the section for tetrahedra and quadrilaterals.
   * **********************************************************
   */
  static Inequality makeX(String c1,String c2, String rhs) {
    return make(new String[]{c1,c2},new Variable[]{t0,d0},CpxProblem.LessEqual,rhs);
  }
  static Inequality makeY(String c1,String c2, String rhs) {
    return make(new String[]{c1,c2},new Variable[]{t1,d1},CpxProblem.LessEqual,rhs);
  }
  static Inequality makeS0(String c1,String c2,String rhs) {
    return make(new String[]{c1,c2},new Variable[]{s0,d0},CpxProblem.LessEqual,rhs);
  }
  static Inequality makeS1(String c1,String c2,String rhs) {
    return make(new String[]{c1,c2},new Variable[]{s1,d1},CpxProblem.LessEqual,rhs);
  }


  static final Variable d0 = makeV("0.8638","1.874445","dihTet");
  static final Variable d1 = makeV("0","6.2832","dihQuad");
  static final Variable t0 = makeV("0","0.6","squanderTet");
  static final Variable t1 = makeV("0","0.6","squanderQuad");
  static final Variable s0 = makeV("-20.0",Parameters.I_pt.max().toString(),"scoreTet");
  static final Variable s1 = makeV("-20.0","0.0","scoreQuad");
  static final Interval minQuad = new Interval("1.153");
  static final Interval twopimax = new Interval("6.2832");
  static final Interval pt = Parameters.I_pt;

  static void addStandard(LPdata data) {
    //tetrahedral relations
    data.addVariable(Parameters.threshold,null);
    Variable t = Parameters.threshold;
    data.addVariable(d0,t);
    data.addVariable(d1,t);
    data.addVariable(t0,t);
    data.addVariable(t1,t);
    data.addVariable(s0,t);
    data.addVariable(s1,t);
    data.addInequality(makeX("-1","-0.3897","-0.4666"));
    data.addInequality(makeX("-1","-0.2993","-0.3683"));
    data.addInequality(makeX("-1","0","0"));
    data.addInequality(makeX("-1","0.1689","0.208"));
    data.addInequality(makeX("-1","0.2529","0.3442"));
    //score tetrahedral:
    data.addInequality(makeS0("1","-0.37898","-0.4111"));
    data.addInequality(makeS0("1","0.142","0.23021"));
    data.addInequality(makeS0("1","0.3302","0.5353"));
    //quad relations
    data.addInequality(makeY("-1","-4.49461","-0.581446"));
    data.addInequality(makeY("-1","-2.1406","-2.955"));
    data.addInequality(makeY("-1","-0.316","-0.6438"));
    data.addInequality(makeY("-1","0","-0.1317"));
    data.addInequality(makeY("-1","0.2365","0.3825"));
    data.addInequality(makeY("-1","0.4747","1.071"));
    //score quad
    data.addInequality(makeS1("1","-4.56766","-5.7906"));
    data.addInequality(makeS1("1","-1.5094","-2.0749"));
    data.addInequality(makeS1("1","-0.5301","-0.8341"));
    data.addInequality(makeS1("1","-0.3878","-0.6284"));
    data.addInequality(makeS1("1","0.1897","0.4124"));
    data.addInequality(makeS1("1","0.5905","1.5707"));
  }

  public static BigDecimal tauLP(int p,int q,Interval angle,String thresh) {
    LPdata data = new LPdata();
    addStandard(data);
    data.addInequality(makeLT(new int[] {p,q},new Variable[] {d0,d1},angle));
    Objective obj = new Objective();
    obj.setSense(CpxProblem.Minimize);
    obj.addCoeff(t0,I(p));
    obj.addCoeff(t1,I(q));
    obj.addCoeff(Parameters.threshold,new Interval(thresh));
    data.setObjective(obj);
    BigDecimal B = null;
    //System.out.println(data.toString());
    try {
      B = data.optimum();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return B;
  }

  public static BigDecimal sigmaLP(int p,int q,Interval angle,String thresh) {

    LPdata data = new LPdata();
    addStandard(data);
    data.addInequality(makeLT(new int[] {p,q},new Variable[] {d0,d1},angle));
    Objective obj = new Objective();
    obj.setSense(CpxProblem.Maximize);
    obj.addCoeff(s0,I(p));
    obj.addCoeff(s1,I(q));
    obj.addCoeff(Parameters.threshold,new Interval(thresh));
    data.setObjective(obj);
    BigDecimal B = null;
    //System.out.println(data.toString());
    try {
      B = data.optimum();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return B;
  }

  public static void runStandardSigma() {
    BigDecimal s5 = new BigDecimal("-0.05704");
    BigDecimal s6 = new BigDecimal("-0.11408");
    BigDecimal s7 = new BigDecimal("-0.17112");
    BigDecimal s8 = new BigDecimal("-0.22816");
    Interval c8pt = new Interval("-3.03").multiply(Parameters.I_pt);
    Interval twopi = Parameters.I_twopi;
    /** s5 + sLP(5,0,2Pi-1.153) < c(8) pt **/ {
      Interval angle = twopi.subtract(new Interval("1.153"));
      BigDecimal sig = sigmaLP(5,0,angle,"-1");
      util.Eiffel.assert(Miscellaneous.lessThan(s5.add(sig),c8pt.min()));
    }
  }

  public static void runStandardTau() {
    BigDecimal target = new BigDecimal("0.819235");
    BigDecimal t8 = new BigDecimal("0.6045");
    BigDecimal t7 = new BigDecimal("0.54999");
    BigDecimal t6 = new BigDecimal("0.41056");
    BigDecimal t5 = new BigDecimal("0.27113");
    BigDecimal t4 = new BigDecimal("0.1317");
    Interval pt = Parameters.I_pt;
    Interval twopi = Parameters.I_twopi;
    Interval d8pt = new Interval("10.916").multiply(pt);
    /* t5 + tLP(5,0,2Pi-1.153) > d(8) pt */ {
          Interval angle = twopi.subtract(new Interval("1.153"));
          BigDecimal tau = tauLP(5,0,angle,"1");
          util.Eiffel.assert(Miscellaneous.lessThan(d8pt.max(),t5.add(tau)));
    }
    /* min tLP(4,0,2Pi-1.32-0.8638), 1.47pt + tLP(4,0,2Pi-1.153-0.8638),
          tLP(5,0,2Pi-1.153) > target-t8 */ {
          BigDecimal pt147 = new Interval("1.47").multiply(pt).min();
          BigDecimal rhs = target.subtract(t8);
          Interval angle1 = twopi.subtract(new Interval("1.32")).subtract(new Interval("0.8638"));
          BigDecimal lhs1 = tauLP(4,0,angle1,"1.0");
          Interval angle2 = twopi.subtract(new Interval("1.153")).subtract(new Interval("0.8638"));
          BigDecimal lhs2 = tauLP(4,0,angle2,"1.0").add(pt147);
          Interval angle3 = twopi.subtract(new Interval("1.153"));
          BigDecimal lhs3 = tauLP(5,0,angle3,"1.0");
          util.Eiffel.assert(Miscellaneous.lessThan(rhs,lhs1));
          util.Eiffel.assert(Miscellaneous.lessThan(rhs,lhs2)," "+lhs2+ " "+rhs);
          util.Eiffel.assert(Miscellaneous.lessThan(rhs,lhs3));
    }
    /* 2t4 + t5 + tLP(3,0,2Pi-3(1.153)) > target */ {
        Interval angle = twopi.subtract(new Interval("1.153").multiply(new BigDecimal("3.0")));
        BigDecimal tau = tauLP(3,0,angle,"1.0");
        BigDecimal t = t4.add(t4).add(t5).add(tau);
        util.Eiffel.assert(Miscellaneous.lessThan(target,t));
    }
    /* 2t5 + tLP(4,0,2Pi-2(1.153)) > target */ {
      Interval angle = twopi.subtract(new Interval("1.153").multiply(new BigDecimal("2.0")));
      BigDecimal tau = tauLP(4,0,angle,"1.0");
      BigDecimal t = t5.add(t5).add(tau);
      util.Eiffel.assert(Miscellaneous.lessThan(target,t));
    }
  }

  public static void tauLPexample() {
    BigDecimal target = new BigDecimal("0.819235");
    BigDecimal t8 = new BigDecimal("0.6045");
    BigDecimal t7 = new BigDecimal("0.54999");
    BigDecimal t6 = new BigDecimal("0.41056");
    BigDecimal t5 = new BigDecimal("0.27113");
    BigDecimal pt = Parameters.I_pt.max();
    BigDecimal B = tauLP(4,0,new Interval(new BigDecimal(2*Math.PI-2*(0.8638)-0*1.32-0*1.153)),"1.0");
    //BigDecimal C = tauLP(4,0,new Interval(new BigDecimal(2*Math.PI-0*(0.8638)-1*1.32-0*1.153)),"1.0");
    System.out.println(B);//.add(t8).subtract(target));
    //System.out.println(new Interval(t8.subtract(t6).add(B)).divide(Parameters.I_pt));
  }

  public static void sigmaLPexample() {
    BigDecimal B = sigmaLP(5,0,new Interval(new BigDecimal(2*Math.PI-0*(0.8638)-0*1.32-1*1.153)),"-100.0");
    //BigDecimal C = tauLP(4,0,new Interval(new BigDecimal(2*Math.PI-0*(0.8638)-1*1.32-0*1.153)),"1.0");
    System.out.println(B);//.add(t8).subtract(target));
    //System.out.println(new Interval(t8.subtract(t6).add(B)).divide(Parameters.I_pt));
  }

  /**
   * ***************************************************
   *                  MAIN
   * ***************************************************
   */
  public static void main(String[] args) {
    //sigmaLPexample();
    //uprightTauLPexample();
    runStandardSigma();
    runStandardTau();
  }

}

