package linprog;

/**
 * Title:        KeplerConjecture 2001
 * Description:  Licensed under GNU GPL http://www.gnu.org/copyleft/gpl.html
 * Copyright:    Copyright (c) 2001, Thomas C. Hales
 *
 * @author Thomas C. Hales
 * @version 2001.0
 */

/**
 * Module of mathematical functions related to solid trigonometry and geometry.
 * All methods are static.
 * This module is a rough equivalent of the kepfn.cpp file in the C++ library.
 * The functions that rely on floating point arithmetic (doubles) should
 * not be used, except heuristically in the proof of the Kepler conjecture.
 */
public class Trig {

  private Trig() {
  }

/**
 * Helper to ABound; Mathematica:  A[h], h = y/2.
 *  A[h,t] = (1 - h/t)*(phi[t, t] - phi[h, t])
 */
public static Interval A(Interval h, Interval t) {
  Interval f = Parameters.I_one.subtract(h.divide(t));  // (1-h/t)
  return f.multiply(phi(t,t).subtract(phi(h,t)));
}

/**
 * Helper for A; Mathematica phi[h,trunc];
 */
public static Interval phi(Interval h, Interval t) {
  // t*h*(h+t)/6 :
  return h.multiply(t).multiply(h.add(t)).divide(Parameters.I_six);
}

  public static class Test extends util.UnitTest { // test values courtesy of Mathematica

  }


}