package linprog;

/**
 * Title:        KeplerConjecture 2001
 * Description:  Licensed under GNU GPL http://www.gnu.org/copyleft/gpl.html
 * Copyright:    Copyright (c) 2001, Thomas C. Hales
 * Company:
 * @author Thomas C. Hales
 * @version 2001.0
 */
import java.math.BigDecimal;

/**
 * This represents a vertex of a graph in a linear programming problem.
 * The constructor contains no parameters, because all the data is held in a workspace.
 * The magnitude of the vertex is the center to vertex distance.
 * <p>
 * @precondition the edge has length at most 4.
 * In a saturated packing, each point of a face of a Voronoi cell has distance
 * at most 2 from the center, so the defining vertex of that face has distance
 * at most 4.
 */
public class Vertex extends Variable {

  private static final BigDecimal two = new BigDecimal(2.0);
  private static final BigDecimal four = new BigDecimal(4.0);

  public Vertex() {
    super(two,four,"yv");
  }

  public Vertex(String id) {
    super(two,four,"yv"+id);
  }

}